@@ -1,7 +1,7 @@
 /**
  * Handles building of classes
  *
- *  Copyright (C) 2011, 2012, 2013, 2014 Mike Gerwitz
+ *  Copyright (C) 2011, 2012, 2013, 2014 Free Software Foundation, Inc.
  *
  *  This file is part of GNU ease.js.
  *
@@ -244,16 +244,14 @@ exports.isInstanceOf = function( type, instance )
         return false;
     }
 
-    try
+    // check prototype chain (will throw an error if type is not a
+    // constructor (function)
+    if ( ( typeof type === 'function' )
+        && ( instance instanceof type )
+    )
     {
-        // check prototype chain (will throw an error if type is not a
-        // constructor (function)
-        if ( instance instanceof type )
-        {
-            return true;
-        }
+        return true;
     }
-    catch ( e ) {}
 
     // if no metadata is available, then our remaining checks cannot be
     // performed
@@ -299,9 +297,10 @@ exports.prototype.build = function extend( _, __ )
     // ensure we'll be permitted to instantiate abstract classes for the base
     this._extending = true;
 
-    var args      = Array.prototype.slice.call( arguments ),
-        props     = args.pop() || {},
-        base      = args.pop() || exports.ClassBase,
+    var a         = arguments,
+        an        = a.length,
+        props     = ( ( an > 0 ) ? a[ an - 1 ] : 0 ) || {},
+        base      = ( ( an > 1 ) ? a[ an - 2 ] : 0 ) || exports.ClassBase,
         prototype = this._getBase( base ),
         cname     = '',
         autoa     = false,
@@ -508,7 +507,10 @@ exports.prototype.buildMembers = function buildMembers(
         {
             handlers[ name ] = function()
             {
-                var args = Array.prototype.slice.call( arguments );
+                var args = [],
+                    i    = arguments.length;
+
+                while ( i-- ) args[ i ] = arguments[ i ];
 
                 // invoke the custom handler with the original handler as
                 // its last argument (which the custom handler may choose
