@@ -1,7 +1,7 @@
 /**
  * Handles building members (properties, methods)
  *
- *  Copyright (C) 2010, 2011, 2012, 2013, 2014 Mike Gerwitz
+ *  Copyright (C) 2010, 2011, 2012, 2013, 2014 Free Software Foundation, Inc.
  *
  *  This file is part of GNU ease.js.
  *
@@ -173,9 +173,12 @@ exports.buildMethod = function(
         }
 
     }
-    else if ( keywords[ 'abstract' ] )
+    else if ( keywords[ 'abstract' ] || keywords[ 'private' ] )
     {
-        // we do not want to wrap abstract methods, since they are not callable
+        // we do not want to wrap abstract methods, since they are not
+        // callable; further, we do not need to wrap private methods, since
+        // they are only ever accessible when we are already within a
+        // private context (see test case for more information)
         dest[ name ] = value;
     }
     else
