@@ -1,7 +1,7 @@
 /**
  * Contains utilities functions shared by modules
  *
- *  Copyright (C) 2010, 2011, 2012, 2013, 2014 Mike Gerwitz
+ *  Copyright (C) 2010, 2011, 2012, 2013, 2014 Free Software Foundation, Inc.
  *
  *  This file is part of GNU ease.js.
  *
@@ -246,6 +246,19 @@ exports.copyTo = function( dest, src, deep )
 };
 
 
+/**
+ * Throw an exception
+ *
+ * Yes, this function has purpose; see where it's used.
+ *
+ * @param  {Error}  e  exception to throw
+ */
+function _throw( e )
+{
+    throw e;
+}
+
+
 /**
  * Parses object properties to determine how they should be interpreted in an
  * Object Oriented manner
@@ -268,6 +281,8 @@ exports.propParse = function( data, options, context )
         callbackGetSet = options.getset        || fvoid,
         keywordParser  = options.keywordParser || propParseKeywords,
 
+        throwf = options._throw || _throw,
+
         hasOwn = Object.prototype.hasOwnProperty,
 
         parse_data = {},
@@ -315,12 +330,12 @@ exports.propParse = function( data, options, context )
 
             if ( !( value instanceof Array ) )
             {
-                throw TypeError(
+                throwf( TypeError(
                     "Missing parameter list for abstract method: " + name
-                );
+                ) );
             }
 
-            verifyAbstractNames( name, value );
+            verifyAbstractNames( throwf, name, value );
             value = exports.createAbstractMethod.apply( this, value );
         }
 
@@ -363,22 +378,24 @@ exports.propParse = function( data, options, context )
  * In the future, we may add additional functionality, so it's important to
  * restrict this as much as possible for the time being.
  *
+ * @param  {function(Error)}  throwf  function to call with error
+ *
  * @param  {string}  name    name of abstract member (for error)
  * @param  {Object}  params  parameter list to check
  *
  * @return {undefined}
  */
-function verifyAbstractNames( name, params )
+function verifyAbstractNames( throwf, name, params )
 {
     var i = params.length;
     while ( i-- )
     {
         if ( params[ i ].match( /^[a-z_][a-z0-9_]*$/i ) === null )
         {
-            throw SyntaxError(
+            throwf( SyntaxError(
                 "Member " + name + " contains invalid parameter '" +
                 params[ i ] + "'"
-            );
+            ) );
         }
     }
 }
@@ -399,7 +416,10 @@ function verifyAbstractNames( name, params )
  */
 exports.createAbstractMethod = function( def )
 {
-    var definition = Array.prototype.slice.call( arguments );
+    var dfn = [],
+        i   = arguments.length;
+
+    while ( i-- ) dfn[ i ] = arguments[ i ];
 
     var method = function()
     {
@@ -407,7 +427,7 @@ exports.createAbstractMethod = function( def )
     };
 
     exports.defineSecureProp( method, 'abstractFlag', true );
-    exports.defineSecureProp( method, 'definition', definition );
+    exports.defineSecureProp( method, 'definition', dfn );
     exports.defineSecureProp( method, '__length', arguments.length );
 
     return method;
@@ -562,8 +582,8 @@ exports.defineSecureProp( exports.getPropertyDescriptor, 'canTraverse',
 
 
 /**
- * Appropriately returns defineSecureProp implementation to avoid check on each
- * invocation
+ * Appropriately returns defineSecureProp implementation to avoid check on
+ * each invocation
  *
  * @return  {function( Object, string, * )}
  */
@@ -584,26 +604,14 @@ function getDefineSecureProp()
         // uses ECMAScript 5's Object.defineProperty() method
         return function( obj, prop, value )
         {
-            try
+            Object.defineProperty( obj, prop,
             {
-                Object.defineProperty( obj, prop,
-                {
-                    value: value,
+                value: value,
 
-                    enumerable:   false,
-                    writable:     false,
-                    configurable: false,
-                });
-            }
-            catch ( e )
-            {
-                // let's not have this happen again, as repeatedly throwing
-                // exceptions will do nothing but slow down the system
-                exports.definePropertyFallback( true );
-
-                // there's an error (ehem, IE8); fall back
-                fallback( obj, prop, value );
-            }
+                enumerable:   false,
+                writable:     false,
+                configurable: false,
+            } );
         };
     }
 }
